/*!
CMouseSimple.h
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#ifndef _CMOUSESIMPLE_H_
#define _CMOUSESIMPLE_H_


/*-----------------------------------------------------------------------------------------------*/
/*!
\brief	Very simple mouse class.
 
 This mouse class is really only used as a data collection point for GLUT functions.
 Only mouse position and button states are stored here, along with some timing for drags and such...
 Created from InitGL() and written to by the glutMotion...() functions, read by any class or object that needs to.
 */
/*-----------------------------------------------------------------------------------------------*/
class CMouseSimple
{
public:
	
	/*!
	Enumeration for some default mouse buttons. Values are assumed to be correct.
	 */
	enum{
		MOUSE_LEFT,
		MOUSE_MIDDLE,
		MOUSE_RIGHT,
		MINIMUM_DRAG_TIME = 250
	};
	
	CMouseSimple();
	~CMouseSimple();
	
	int GetNumButtons(void) const {	return m_NumButtons; }	//!< Get the number of buttons on the mouse.
	bool GetButton(int button);								//!< Get the state of a button.
	bool SetButton(int button, bool state);					//!< Set the statet of a button.

	inline int GetMouseXi(void) const { return m_X; }		//!< Get mouse X coord as an int in screen space.
	inline int GetMouseYi(void) const { return m_Y; }		//!< Get mouse Y coord as an int in screen space.
	inline float GetMouseXf(void) const { return m_fX; }	//!< Get mouse X coord as a float in screen space.
	inline float GetMouseYf(void) const { return m_fY; }	//!< Get mouse Y coord as a float in screen space.
	void SetMouseXY(int x, int y);							//!< Set mouse screen coordinate (from GLUT function).

	int GetDragTime(int button);							//!< Get the length of time a mouse button has been down.
	bool ButtonClick(int button);							//!< Check if a button has just been clicked.
	void UnClickButton(int button);							//!< Reset a clicked button.
	void SetLimitRegion(int x1, int y1, int x2, int y2);	//!< Set a region to limit the mouse into.
	bool LimitMouseToRegion(void);							//!< Limit the mouse to a previously set limit region.
	void SetScreenSize(int w, int h);						//!< Set the screen size, required for float value calculations.

private:
	float m_fX;					//!< Calculated X float coord of mouse.
	float m_fY;					//!< Calculated Y float coord of mouse. 
	int m_X;					//!< X coord of mouse in screenspace.
	int m_Y;					//!< Y coord of mouse in screenspace. 
	int m_NumButtons;			//!< Number of buttons on the connected mouse.
	bool *m_Buttons;			//!< Button states, allocated in constructor.
	time_t *m_DragTime;			//!< Drag timers for each button. Only first few are usefull.
	int m_iDisplayWidth;		//!< Pixel width of the screen or window.
	int m_iDisplayHeight;		//!< Pixel height of the screen or window.
	int m_iDisplayWidthHalf;	//!< Half pixel width of the screen or window.
	int m_iDisplayHeightHalf;	//!< half pixel height of the screen or window.
	int m_Limit[4];				//!< Limit region for mouse.
};


/*-----------------------------------------------------------------------------------------------*/
/*!
A global CMouseSimple object for access by all.
 Needs to be created at program "start" befoire use.
 */
/*-----------------------------------------------------------------------------------------------*/
extern CMouseSimple *g_pMouseSimple;


#endif
//END
